/***********************************************************************
#
#		aevt.c
#
#		Apple events handler.
#
#		Author: Michael Marinkovich
#				Apple Developer Technical Support
#
#
#		Modification History: 
#
#			6/4/95		MWM 	Initial coding					 
#			10/12/95	MWM		cleaned up
#
#		Copyright  1992-95 Apple Computer, Inc., All Rights Reserved
#
#
***********************************************************************/

#include <AppleEvents.h>
#include <Windows.h>

#include "App.h"
#include "Proto.h"


extern Boolean		gDone;


//----------------------------------------------------------------------
//
//	AEInit - initialize all the core apple events
//
//
//----------------------------------------------------------------------

OSErr AEInit(void)
{	
	OSErr		err = noErr;
					
								//	install auto Open App
	err = AEInstallEventHandler(kCoreEventClass, kAEOpenApplication, 
								NewAEEventHandlerProc(DoAEOpenApp), 0L, false );
													
	if (err == noErr)			// install auto Quit App
		err = AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, 
									NewAEEventHandlerProc(DoAEQuitApp), 0L, false );
		
	if (err == noErr)			// install auto Open Document
		err = AEInstallEventHandler(kCoreEventClass,kAEOpenDocuments,
									NewAEEventHandlerProc(DoAEOpenDoc),0L,false);
														
	if (err == noErr)			// install auto Print Document
		err = AEInstallEventHandler(kCoreEventClass,kAEPrintDocuments,
									NewAEEventHandlerProc(DoAEPrintDoc),0L,false);
	
							
	return err;
												
}


//----------------------------------------------------------------------
//
//	DoAEOpenApp - called by the Finder at launch time
//
//
//----------------------------------------------------------------------

pascal OSErr DoAEOpenApp(AppleEvent event,AppleEvent reply,long refCon)
{
	#pragma unused( event, reply, refCon )

	(void)CreateWindow(128, nil, nil, nil, true,
						documentProc, kDocKind, (WindowPtr)-1, true, nil);
						
	return noErr;
		
}


//----------------------------------------------------------------------
//
//	 DoAEQuitApp -	called when the Finder asks app to quit
//
//
//----------------------------------------------------------------------

pascal OSErr DoAEQuitApp(AppleEvent event,AppleEvent reply,long refCon)
{
	#pragma unused( event, reply, refCon )

	// set the global quit flag
	gDone = true;
	
	return noErr;
		
}


//----------------------------------------------------------------------
//
//	DoAEOpenDoc - called when the Finder asks app to open a document
//
//
//----------------------------------------------------------------------

pascal OSErr DoAEOpenDoc(AppleEvent event,AppleEvent reply,long refCon)
{
	#pragma unused( reply, refCon )
	

	return noErr;

}
				
										
//----------------------------------------------------------------------
//
//	DoAEPrintDoc - called when the Finder asks app to print a document
//
//
//----------------------------------------------------------------------

pascal OSErr DoAEPrintDoc(AppleEvent event,AppleEvent reply,long refCon)
{
	#pragma unused( reply, refCon )
	

	return noErr;

}


//----------------------------------------------------------------------
//
//	GotAEParams - make sure we got proper AE params for an Open 
//				  Document from the Finder
//
//----------------------------------------------------------------------

OSErr GotAEParams(AppleEvent *appleEvent)
{
	OSErr			err;
	DescType		type;
	Size			size;
	
	err = AEGetAttributePtr(appleEvent,keyMissedKeywordAttr,
							typeWildCard,&type,nil,0,&size);
											
	if (err == errAEDescNotFound)
		return(noErr);
	
	else
		if (err == noErr)
			return(errAEEventNotHandled);
	
	return err;

} 
