/***********************************************************************
#
#		init.c
#
#		basic initialization code.
#
#		Author: Michael Marinkovich
#				Apple Developer Technical Support
#
#
#		Modification History: 
#
#			6/4/95		MWM 	Initial coding					 
#			10/12/95	MWM		cleaned up
#
#		Copyright  1992-95 Apple Computer, Inc., All Rights Reserved
#
#
***********************************************************************/


#include <AppleEvents.h>
#include <Displays.h>
#include <Events.h>
#include <Fonts.h>
#include <Gestalt.h>
#include <Menus.h>
#include <OSUtils.h>

#include "App.h"
#include "Proto.h"

extern Boolean 			gHasDMTwo;
extern Boolean 			gHasDrag;
extern Boolean			gInBackground;
extern short			gWindCount;


//----------------------------------------------------------------------
//
//	Initialize - the main entry point for the initialization
//
//
//----------------------------------------------------------------------

OSErr Initialize(void)
{
	OSErr		err = noErr;
	
	
	ToolBoxInit();
	CheckEnvironment();
	err = InitApp();
	
	return err;
}


//----------------------------------------------------------------------
//
//	ToolBoxInit - initialization all the needed managers
//
//
//----------------------------------------------------------------------

void ToolBoxInit(void)
{

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
		
	FlushEvents(everyEvent, 0);

}


//----------------------------------------------------------------------
//
//	CheckEnvironment - make sure we can run with current sys and managers.
//					   Also initialize globals - have drag & drop
//					  
//----------------------------------------------------------------------

void CheckEnvironment(void)
{
	OSErr			err;
	long			response;
	

	// check to see if the Display Manager is present. If it is 
	// then we have a PowerMac or System 7.5.
	err = Gestalt(gestaltDisplayMgrAttr, &response);
	
	if (err != noErr || ((response & (1 << gestaltDisplayMgrPresent)) == 0))
	 	HandleAlert(kNeedsDisplayManager);
	
	// check for Display Manager 2.0
	else {
		Gestalt(gestaltDisplayMgrVers, (long*)&response);
		gHasDMTwo = (response >= 0x00020000);
	}

}


//----------------------------------------------------------------------
//
//	InitApp - initialization all the application specific stuff
//
//
//----------------------------------------------------------------------

OSErr InitApp(void)
{
	OSErr				err;

	// init AppleEvents
	err = AEInit();
	MenuSetup();

	// we have DM 2.0 so install
	// world changed notification proc
	if (gHasDMTwo)
		err = InstallDMNotification();

	// we have DM 1.0 so install AppleEvent Notification
	else
		err = InstallAEDMNotification();

	// init any globals
	gWindCount = 1;

	return err;					
}


//----------------------------------------------------------------------
//
//	MenuSetup - 
//
//
//----------------------------------------------------------------------

void MenuSetup(void)
{
	Handle			menu;
		
		
	menu = GetNewMBar(rMBarID);		//	get our menus from resource
	SetMenuBar(menu);
	DisposeHandle(menu);
	AddResMenu(GetMHandle(mApple ), 'DRVR');		//	add apple menu items

	DrawMenuBar();

		
}
