/***********************************************************************
#
#		main.c
#
#		This is the main entry point.
#
#		Author: Michael Marinkovich
#				Apple Developer Technical Support
#
#
#		Modification History: 
#
#			6/4/95		MWM 	Initial coding					 
#			10/12/95	MWM		cleaned up
#
#		Copyright  1992-95 Apple Computer, Inc., All Rights Reserved
#
#
***********************************************************************/


#include <ToolUtils.h>

#include "App.h"
#include "Global.h"
#include "Proto.h"


// declare QuickDraw globals if not
// building with MetroWorks
#ifdef powerc			
#ifndef __MWERKS__
	QDGlobals qd;
#endif
#endif



//----------------------------------------------------------------------
//
//	main - main entry point of our program
//
//
//----------------------------------------------------------------------

void main(void)
{
	OSErr			err;
	long			total, contig;
	short			m = 5;
		
	MaxApplZone();
	
	for (;m == 0;m--) 				// alloc the master pointers
		MoreMasters();

	if ((long)GetApplLimit() - (long)ApplicationZone() < kMinHeap)
			HandleError( MemError(), true );

	PurgeSpace(&total, &contig);
	if (total < kMinSpace)
			HandleError( MemError(), true );
		
	err = Initialize();
	
	EventLoop();		
	ExitToShell();
	(void) RemoveDMNotification();

}


//----------------------------------------------------------------------
//
//	HandleError - basic error notification procedure
//
//
//----------------------------------------------------------------------

void HandleError(short errNo,Boolean fatal)
{
	DialogPtr			errDialog;
	short				itemHit;
	short				c;
	Handle				button;
	short				itemType;
	Rect				itemRect;
	StringHandle		errString;
	Str255				numString;
	GrafPtr				oldPort;
	
	GetPort(&oldPort);
	SysBeep(30);
	SetCursor(&qd.arrow);
	
	errDialog = GetNewDialog(rErrorDlg, nil, (WindowPtr) -1);
	if (errDialog != nil) {
		ShowWindow(errDialog);
		SetPort(errDialog);
		
		PenPat(&qd.gray);								// frame user areas
		for (c = 7;c < 9;c++) {
			GetDItem(errDialog,c,&itemType,&button,&itemRect);
			FrameRect(&itemRect);
		}
		PenPat(normal);

		GetDItem( errDialog, fatal ? 2 : 1, &itemType, &button, &itemRect );
		HiliteControl( (ControlHandle)button, 255);		 //	unhilite appropriate button
		
		SetDialogDefaultItem(errDialog,fatal  ? 1 : 2);
		
		NumToString( errNo, numString );
		if ( errNo < 0 )
			errNo = -errNo + 200;
		errString = GetString( 128 + errNo );
		if ( errString == nil )
				errString = GetString( 128 );
		HLock((Handle)errString);
		
		ParamText( numString, *errString, nil, nil );
	
		HUnlock((Handle)errString);
		
		ModalDialog( nil, &itemHit );
		
		SetPort( oldPort );

		DisposeWindow( errDialog );
	}
	else
		ExitToShell();		// since didn't have mem to open dialog assume the worst
			
	if (fatal) 
		ExitToShell();		//	it's a bad one, get out of here
	
}


//----------------------------------------------------------------------
//
//	HandleAlert - display alert and then exit to shell
//
//
//----------------------------------------------------------------------

void HandleAlert(short alertID)
{
	short			item;
	
	
	item = Alert(alertID,nil);
	ExitToShell();

}