#include <FixMath.h>
#include "QD3DtoQTVR.h"
#include "extern.h"
#include "camera.h"

TQ3CameraObject MyNewCamera(CWindowPtr theWindow)
{
	TQ3ViewAngleAspectCameraData	perspectiveData;
	TQ3CameraObject					camera;
	// Set the field of view to 74 degrees (or 74*kQ3Pi/180 radians).
	float 							fieldOfView =  74.0*kQ3Pi/180.0;
	TQ3Status						returnVal = kQ3Failure ;

	// Assign default placement and range
	perspectiveData.cameraData.placement.cameraLocation 	= kMyDefaultFrom;
	perspectiveData.cameraData.placement.pointOfInterest 	= kMyDefaultTo;
	perspectiveData.cameraData.placement.upVector 			= kMyDefaultUp;
	perspectiveData.cameraData.range.hither	= kMyDefaultHither;
	perspectiveData.cameraData.range.yon 	= kMyDefaultYon;

	// Assign standard viewport
	perspectiveData.cameraData.viewPort.origin.x = -1.0;
	perspectiveData.cameraData.viewPort.origin.y = 1.0;
	perspectiveData.cameraData.viewPort.width = 2.0;
	perspectiveData.cameraData.viewPort.height = 2.0;
	
	perspectiveData.fov				= fieldOfView;
	perspectiveData.aspectRatioXToY	=
		(float) (theWindow->portRect.right - theWindow->portRect.left) / 
		(float) (theWindow->portRect.bottom - theWindow->portRect.top);
		
	camera = Q3ViewAngleAspectCamera_New(&perspectiveData);
	
	return camera ;
}

void MyGetBoundingSphere(TQ3ViewObject viewObject, TQ3GroupObject mainGroup,
					 TQ3BoundingSphere *viewBSphere)
{
	TQ3Status					status;
	
	Q3View_StartBoundingSphere(viewObject,kQ3ComputeBoundsApproximate);
	do {
		status = Q3DisplayGroup_Submit(mainGroup, viewObject);
	} while (Q3View_EndBoundingSphere(viewObject, viewBSphere) == kQ3ViewStatusRetraverse);								        
}

void MyInitPanoCamera(DocumentPtr theDocument)
{
	TQ3BoundingSphere 			viewBSphere;
	float						viewRadius;
	TQ3Point3D					zeroPoint	= {0,0,0};
	
	if(theDocument == NULL)
		return;
		
	// Get the bounding sphere of the drawable group (the entire model) in the view.
	MyGetBoundingSphere(theDocument->theView, theDocument->documentGroup, &viewBSphere);

	// Get the bounding sphere's center and radius
	theDocument->documentGroupCenter = viewBSphere.origin;
	viewRadius = viewBSphere.radius;
	
	// Center the camera.
	MyPlaceCamera(theDocument, theDocument->documentGroupCenter.x,
				theDocument->documentGroupCenter.y,
				theDocument->documentGroupCenter.z,
				theDocument->documentGroupCenter.x,
				theDocument->documentGroupCenter.y,
				theDocument->documentGroupCenter.z+1.0);
}


void	MyRotateCameraY(DocumentPtr theDocument, float dY)
{
	TQ3CameraObject		camera;
	TQ3CameraPlacement	cameraPos;
	TQ3Matrix4x4		myRotation;
	TQ3Vector3D			initialVector,rotatedVector;
	
	Q3View_GetCamera(theDocument->theView, &camera );
	Q3Camera_GetPlacement(camera, &cameraPos);
	
	// Get Z Vector
	Q3Point3D_Subtract(&cameraPos.pointOfInterest,&cameraPos.cameraLocation,&initialVector);
	
	// Get rotation about Y axis.
	Q3Matrix4x4_SetRotateAboutAxis(&myRotation,&cameraPos.cameraLocation,&cameraPos.upVector,dY);
	
	// Rotate Z vector about Y axis.
	Q3Vector3D_Transform(&initialVector,&myRotation,&rotatedVector);
	
	// Get Point of Interest from Rotated vector. The upVector does not change.
	Q3Point3D_Vector3D_Add(&cameraPos.cameraLocation,&rotatedVector,&cameraPos.pointOfInterest);
	
	Q3Camera_SetPlacement(camera, &cameraPos);
	Q3View_SetCamera(theDocument->theView, camera );
	Q3Object_Dispose(camera);
}


void	MyPlaceCamera(DocumentPtr theDocument, float fromX,float fromY,float fromZ,
				float toX,float toY,float toZ)
{
	TQ3CameraObject		camera;
	TQ3CameraPlacement	cameraPos;

	Q3View_GetCamera(theDocument->theView, &camera );
	Q3Camera_GetPlacement(camera, &cameraPos);
	
	cameraPos.cameraLocation.x = fromX;
	cameraPos.cameraLocation.y = fromY;
	cameraPos.cameraLocation.z = fromZ;
	
	cameraPos.pointOfInterest.x = toX;
	cameraPos.pointOfInterest.y = toY;
	cameraPos.pointOfInterest.z = toZ;
	
	Q3Camera_SetPlacement(camera, &cameraPos);
	Q3View_SetCamera(theDocument->theView, camera );
	Q3Object_Dispose(camera);
}




