/*
 *  ToolFrontEndPanel.c - ToolServer Front End Drop-In Preferences
 */
 

#define	BUILDING_FOR_SYSTEM7	1

/* standard headers */
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

/* system headers */
#include <AERegistry.h>
#include <Resources.h>
#include <TextUtils.h>
#include <Controls.h>
#include <LowMem.h>

/* compiler headers */
#include <A4Stuff.h>
#include <SetUpA4.h>
#include <DropInPanel.h>

/* project headers */
#include "ToolFrontEnd.h"
#include "MoreFilesExtras.h"


/* current version number for our prefs data */
#define PTOOLFRONTENDVERSION 2


enum {
	kItemListID = 128,
	kIncludeScannerItem = 1,
	kIncludePathItem = 2,
	kIncludeBoxItem = 5,
	kCommandLabelItem = 6,
	kCommandItem = 7,
	kExtensionPopupItem = 8,
	kScriptIncludeFileItem = 10,
	kOverallBoxItem = 11,
	kGetExtensionDialog = 129,
	kExtensionTextItem = 4
};


typedef struct
{
	short current;				// current selection in popup menu
	Boolean enableIncludes;		// scan for include files
	Boolean popupsInitialized;	// has the popup been initialized?
} ToolFrontEndPanelStatus;


/* prototypes of local functions */
static void*	InitDialog(DialogPtr dp, short baseItems);
static void		TermDialog(DialogPtr dp, short baseItems, void *storage);
static OSErr	OptionsToDialog(DialogPtr dp, Handle options, short baseItems, void *storage);
static short	DialogToOptions(DialogPtr dp, Handle options, short baseItems, void *storage);
static short	Filter(DialogPtr dp, EventRecord *event, short *itemHit, void *storage);
static void		ItemHit(PanelParameterBlock *pb, void *storage);
static void		Validate(Handle original, Handle current, Boolean *recompile, Boolean *relink, Boolean *reset);
static short	GetPref(AEKeyword keyword, AEDesc *prefsDesc, Handle settings);
static short	SetPref(AEKeyword keyword, const AEDesc *prefsDesc, Handle settings);
static short	GetFactory(Handle settings, DialogPtr dp);
static short	UpdatePref(Handle settings);
static Boolean	ComparePrefs(Handle prefsHand1, Handle prefsHand2);
static pascal void	PanelDrawIncludeBox(DialogPtr dp, short item);
static pascal void	PanelDrawOverallBox(DialogPtr dp, short item);
static Boolean	GetExtensionString(StringPtr extension);
static void		EnableMainItems(DialogPtr dp, short baseItems);
static void		DisableAllItems(DialogPtr dp, short baseItems);
static void		EnableIncludeItems(DialogPtr dp, short baseItems);
static void		DisableIncludeItems(DialogPtr dp, short baseItems);
static void		FixEditField(DialogPtr dp);


static void *theStorage = NULL; // for DrawUserItem

/*
 *	main	-	entry-point for Drop-In Preferences Panel
 *
 */
 
pascal short main(PanelParameterBlock *pb)
{
	short err = noErr;
	
	EnterCodeResource();
	PrepareCallback();
	
	switch (pb->request)
	{
	case reqInitPanel:
		/* panel has just been loaded into memory */
		theStorage = NULL;
		break;

	case reqTermPanel:
		/* panel is about to be unloaded from memory */
		break;

	case reqInitDialog:
		/* hook our dialog item list into the preferences dialog */
		theStorage = pb->storage = InitDialog(pb->dialog, pb->baseItems);
		break;
		
	case reqTermDialog:
		/* unhook our dialog item list from the preferences dialog */
		TermDialog(pb->dialog, pb->baseItems, pb->storage);
		break;
	
	case reqPutData:
		/* put the data in the given handle into our dialog items */
		OptionsToDialog(pb->dialog, pb->currentPrefs, pb->baseItems, pb->storage);
		break;

	case reqGetData:
		/* fill in the given handle with our dialog items */
		err = DialogToOptions(pb->dialog, pb->currentPrefs, pb->baseItems, pb->storage);
		break;

	case reqFilter:
		/* filter an event in the dialog */
		err = Filter(pb->dialog, pb->event, &pb->itemHit, pb->storage);
		break;
		
	case reqItemHit:
		/* handle a hit on one of our dialog items */
		ItemHit(pb, pb->storage);
		break;
		
	case reqAEGetPref:
		/* return one item in the given handle as an Apple Event descriptor */
		err = GetPref(pb->prefsKeyword, &pb->prefsDesc, pb->currentPrefs);
		break;

	case reqAESetPref:
		/* change one item in the given handle according to the given Apple Event descriptor */
		err = SetPref(pb->prefsKeyword, &pb->prefsDesc, pb->currentPrefs);
		break;

	case reqValidate:
		/* determine if we need to reset paths, recompile, or relink */
		Validate(pb->originalPrefs, pb->currentPrefs, &pb->recompile, &pb->relink, &pb->reset);
		break;

	case reqGetFactory:
		/* return our factory settings */
		err = GetFactory(pb->factoryPrefs, pb->dialog);
		break;

	case reqUpdatePref:
		/* update the given handle to use the current format for our prefs data */
		err = UpdatePref(pb->currentPrefs);
		break;
		
	default:
		err = paramErr;
		break;
	}
	
	ExitCodeResource();
	
	return err;
}


/*
 *	PanelDrawOverallBox  -	user item proc to draw box
 */
 
static pascal void PanelDrawOverallBox(DialogPtr dp, short item)
{
	GrafPtr		savedPort;
	PenState	savedPen;
	Handle		itemHand;
	short		itemType;
	Rect		itemRect;
	ToolFrontEndPanelStatus** status;
	
	EnterCallback();
	
	status = (ToolFrontEndPanelStatus**)theStorage;
	
	GetPort(&savedPort);
	SetPort((GrafPtr)dp);
	GetPenState(&savedPen);
	PenNormal();
	
	GetDialogItem(dp, item, &itemType, &itemHand, &itemRect);
	FrameRect(&itemRect);
	
	if (status != NULL && (*status)->current < 0)
	{
		// grey it out -- no extensions yet
		Pattern gray;
		short baseItems = item - kOverallBoxItem;
		Rect labelRect;
		
		GetDialogItem(dp, baseItems + kCommandLabelItem, &itemType, &itemHand, &labelRect);
		
		GetIndPattern(&gray, sysPatListID, 4);
		PenMode(patBic);
		PenPat(&gray);
		InsetRect(&itemRect, 1, 1);
		itemRect.top = labelRect.top - 1;
		PaintRect(&itemRect);
	}
	
	SetPenState(&savedPen);
	SetPort(savedPort);
	
	ExitCallback();
}


/*
 *	PanelDrawIncludeBox  -	user item proc to draw box
 */
 
static pascal void PanelDrawIncludeBox(DialogPtr dp, short item)
{
	GrafPtr		savedPort;
	PenState	savedPen;
	Handle		itemHand;
	short		itemType;
	Rect		itemRect;
	ToolFrontEndPanelStatus** status;
	
	EnterCallback();
	
	status = (ToolFrontEndPanelStatus**)theStorage;
	
	GetPort(&savedPort);
	SetPort((GrafPtr)dp);
	GetPenState(&savedPen);
	PenNormal();
	
	GetDialogItem(dp, item, &itemType, &itemHand, &itemRect);
	FrameRect(&itemRect);
	
	if (status != NULL && (*status)->current >= 0 && !(*status)->enableIncludes)
	{
		// grey it out -- no include options
		Pattern gray;
		GetIndPattern(&gray, sysPatListID, 4);
		PenMode(patBic);
		PenPat(&gray);
		InsetRect(&itemRect, 1, 1);
		PaintRect(&itemRect);
	}
	
	SetPenState(&savedPen);
	SetPort(savedPort);
	
	ExitCallback();
}


/*
 *	InitDialog  -	initialize Dialog Box items for this panel
 *
 */

static void *InitDialog(DialogPtr dp, short baseItems)
{
	ToolFrontEndPanelStatus**	status;
	Handle						itemHand, newItems;
	short						itemType;
	Rect						itemRect;
	UserItemUPP					drawBoxUPP;
	
	newItems = Get1Resource('DITL', kItemListID);
	AppendDITL(dp, newItems, overlayDITL);
	
	drawBoxUPP = NewUserItemProc(PanelDrawIncludeBox);
	GetDialogItem(dp, baseItems + kIncludeBoxItem, &itemType, &itemHand, &itemRect);
	SetDialogItem(dp, baseItems + kIncludeBoxItem, itemType, (Handle) drawBoxUPP, &itemRect);
	
	drawBoxUPP = NewUserItemProc(PanelDrawOverallBox);
	GetDialogItem(dp, baseItems + kOverallBoxItem, &itemType, &itemHand, &itemRect);
	SetDialogItem(dp, baseItems + kOverallBoxItem, itemType, (Handle) drawBoxUPP, &itemRect);
	
	SelectDialogItemText(dp, baseItems + kCommandItem, 0, 32767);
	
	status = (ToolFrontEndPanelStatus**)NewHandleClear(sizeof(ToolFrontEndPanelStatus));
	(*status)->current = -1;

	return (void*)status;
}

/*
 *	TermDialog	-	destroy Dialog Box items for this panel
 *
 */

static void TermDialog(DialogPtr dp, short baseItems, void *storage)
{
	short		itemType;
	Rect		itemRect;
	UserItemUPP	drawBoxUPP;
	ToolFrontEndPanelStatus** status = (ToolFrontEndPanelStatus**)storage;
	
	GetDialogItem(dp, baseItems + kIncludeBoxItem, &itemType, (Handle *) &drawBoxUPP, &itemRect);
	DisposeRoutineDescriptor(drawBoxUPP);
	GetDialogItem(dp, baseItems + kOverallBoxItem, &itemType, (Handle *) &drawBoxUPP, &itemRect);
	DisposeRoutineDescriptor(drawBoxUPP);
	
	if (status != NULL) DisposeHandle((Handle)status);
	theStorage = NULL;
}


/*
 *	OptionsToDialog		-	copy the options data from the handle to the screen
 *
 */

static OSErr OptionsToDialog(DialogPtr dp, Handle options, short baseItems, void *storage)
{
	ToolFrontEndPanelStatus**	status = (ToolFrontEndPanelStatus**)storage;
	ToolFrontEndPrefHandle		prefsData = (ToolFrontEndPrefHandle) options;
	Handle						itemHand;
	short						itemType;
	Rect						itemRect;
	Byte						state;
	int							i;
	MenuHandle					menu;
	ControlHandle				popup;
	Handle						controlData;
	PopupPrivateDataHandle		popupData;
	OSErr						err;

	// initialize the popup menus
	if (!(*status)->popupsInitialized)
	{
		// initialize the filename extensions popup
		GetDialogItem(dp, baseItems + kExtensionPopupItem, &itemType, &itemHand, &itemRect);
		popup = (ControlHandle)itemHand;
		controlData = (*popup)->contrlData;
		popupData = (PopupPrivateDataHandle)controlData;
		menu = (*popupData)->mHandle;
		DisableItem(menu, 2); // disable remove
		while (CountMenuItems(menu) > 2) DeleteMenuItem(menu, 3);
		SetControlMaximum(popup, 2);
		(*status)->current = -1;
	
		for (i = 0; i < (*prefsData)->numExtensions; i++)
		{
			if (i == 0)
			{
				AppendMenu(menu, "\p(-");	// separator line
				EnableItem(menu, 2);		// enable remove
				(*status)->current = 0;
				SetControlMaximum(popup, 3);
			}
			
			state = HGetState(options);
			HLock(options);
			AppendMenu(menu, "\pgrog");
			SetMenuItemText(menu, CountMenuItems(menu), (*prefsData)->extensions[i].extension);
			SetControlMaximum(popup, GetControlMaximum(popup) + 1);
			HSetState(options, state);
			
			if (i == 0)
				SetControlValue(popup, 4);	// select first extension
		}
		
		// initialize the include file scanner popup
		GetDialogItem(dp, baseItems + kIncludeScannerItem, &itemType, &itemHand, &itemRect);
		popup = (ControlHandle)itemHand;
		controlData = (*popup)->contrlData;
		popupData = (PopupPrivateDataHandle)controlData;
		menu = (*popupData)->mHandle;
		
		// find the include file scanner folder
		{
			Str31 folderName;
			FCBPBRec pb;
			CInfoPBRec ci;
			FSSpec *scanners;
			short actualCount;
			short index;
			
			pb.ioFCBIndx = 0; // use ioRefNum
			pb.ioRefNum = LMGetCurApRefNum();
			pb.ioNamePtr = NULL;
			err = PBGetFCBInfoSync(&pb);
			if (err != noErr) return err;
			GetIndString(folderName, 128, 1); // CodeWarrior Plugins
			ci.dirInfo.ioNamePtr = folderName;
			ci.dirInfo.ioVRefNum = pb.ioFCBVRefNum;
			ci.dirInfo.ioDrDirID = pb.ioFCBParID;
			ci.dirInfo.ioFDirIndex = 0;
			err = PBGetCatInfoSync(&ci);
			if (err != noErr) return err;
			GetIndString(folderName, 128, 2); // Include Scanners

			// iterate over all files in folder, adding to popup menu
			scanners = (FSSpec*)NewPtr(sizeof(FSSpec[100]));
			if (scanners == NULL) return MemError();
			index = 1;
			do
			{
				short i;
				err = GetDirItems(pb.ioFCBVRefNum, ci.dirInfo.ioDrDirID, folderName, true, false, scanners, 100, &actualCount, &index);
				for (i = 0; i < actualCount; i++)
				{
					AppendMenu(menu, "\pgrog");
					SetMenuItemText(menu, CountMenuItems(menu), scanners[i].name);
					SetControlMaximum(popup, GetControlMaximum(popup) + 1);
				}
			} while (err == noErr);
			DisposePtr((Ptr)scanners);
		}
		
		(*status)->popupsInitialized = true;
	}

	state = HGetState((Handle)status);
	HLock((Handle)status);
		
	GetDialogItem(dp, baseItems + kScriptIncludeFileItem, &itemType, &itemHand, &itemRect);
	SetDialogItemText(itemHand, (*prefsData)->scriptIncludeFile);

	if ((*status)->current < 0)
	{
		// no current extension -- disable all items
		DisableAllItems(dp, baseItems);
		(*status)->enableIncludes = false;
	}
	else
	{
		EnableMainItems(dp, baseItems);
		(*status)->enableIncludes = ((*prefsData)->extensions[(*status)->current].extension[0] != 0);
		if ((*status)->enableIncludes)
			EnableIncludeItems(dp, baseItems);
		else
			DisableIncludeItems(dp, baseItems);
		
		// set up scanner popup menu selection
		{
			Str31 theScanner;
			short i, lim;
			
			GetDialogItem(dp, baseItems + kIncludeScannerItem, &itemType, &itemHand, &itemRect);
			popup = (ControlHandle)itemHand;
			controlData = (*popup)->contrlData;
			popupData = (PopupPrivateDataHandle)controlData;
			menu = (*popupData)->mHandle;
			BlockMoveData((*prefsData)->extensions[(*status)->current].scanner, theScanner,
						  (*prefsData)->extensions[(*status)->current].scanner[0] + 1);
			for (i = 3, lim = CountMenuItems(menu); i <= lim; i++)
			{
				// if this is the scanner, then select it
				Str31 thisScanner;
				GetMenuItemText(menu, i, thisScanner);
				if (EqualString(thisScanner, theScanner, false, true))
				{
					SetControlValue(popup, i);
					break;
				}
			}
		}
		
		GetDialogItem(dp, baseItems + kIncludePathItem, &itemType, &itemHand, &itemRect);
		SetDialogItemText(itemHand, (*prefsData)->extensions[(*status)->current].includeFileArg);
		
		GetDialogItem(dp, baseItems + kCommandItem, &itemType, &itemHand, &itemRect);
		SetDialogItemText(itemHand, (*prefsData)->extensions[(*status)->current].commandTemplate);
	}
	
	HSetState((Handle)status, state);
	return noErr;
}


/*
 *	DialogToOptions		-	copy the options data from screen to the handle
 *
 */

static short DialogToOptions(DialogPtr dp, Handle options, short baseItems, void *storage)
{
	ToolFrontEndPanelStatus**	status = (ToolFrontEndPanelStatus**)storage;
	ToolFrontEndPref **			prefsData = (ToolFrontEndPrefHandle) options;
	Handle						itemHand;
	short						itemType;
	Rect						itemRect;
	Str255						tempStr;
	Byte						state;
	MenuHandle					menu;
	ControlHandle				popup;
	Handle						controlData;
	PopupPrivateDataHandle		popupData;
	
	state = HGetState((Handle)status);
	HLock((Handle)status);
	
	GetDialogItem(dp, baseItems + kScriptIncludeFileItem, &itemType, &itemHand, &itemRect);
	GetDialogItemText(itemHand, tempStr);
	if (StrLength(tempStr) >= sizeof((*prefsData)->scriptIncludeFile))
		tempStr[0] = sizeof((*prefsData)->scriptIncludeFile) - 1;
	BlockMoveData(tempStr, (*prefsData)->scriptIncludeFile, StrLength(tempStr)+1);
	
	if ((*status)->current < 0)
	{
		// no current extension
	}
	else
	{
		// extract data from scanner popup menu
		GetDialogItem(dp, baseItems + kIncludeScannerItem, &itemType, &itemHand, &itemRect);
		popup = (ControlHandle)itemHand;
		controlData = (*popup)->contrlData;
		popupData = (PopupPrivateDataHandle)controlData;
		menu = (*popupData)->mHandle;
		if (GetControlValue(popup) == 1)
		{
			(*prefsData)->extensions[(*status)->current].scanner[0] = 0;
		}
		else
		{
			Str31 scanner;
			GetMenuItemText(menu, GetControlValue(popup), scanner);
			BlockMoveData(scanner, (*prefsData)->extensions[(*status)->current].scanner, StrLength(scanner)+1);
		}
		
		GetDialogItem(dp, baseItems + kIncludePathItem, &itemType, &itemHand, &itemRect);
		GetDialogItemText(itemHand, tempStr);
		if (StrLength(tempStr) >= sizeof((*prefsData)->extensions[(*status)->current].includeFileArg))
			tempStr[0] = sizeof((*prefsData)->extensions[(*status)->current].includeFileArg) - 1;
		BlockMoveData(tempStr, (*prefsData)->extensions[(*status)->current].includeFileArg, StrLength(tempStr)+1);
		
		GetDialogItem(dp, baseItems + kCommandItem, &itemType, &itemHand, &itemRect);
		GetDialogItemText(itemHand, tempStr);
		if (StrLength(tempStr) >= sizeof((*prefsData)->extensions[(*status)->current].commandTemplate))
			tempStr[0] = sizeof((*prefsData)->extensions[(*status)->current].commandTemplate) - 1;
		BlockMoveData(tempStr, (*prefsData)->extensions[(*status)->current].commandTemplate, StrLength(tempStr)+1);
	}
	
	HSetState((Handle)status, state);

	return noErr;
}

/*
 *	Filter		-	filter an event for the Preferences panel
 *
 */

static short Filter(DialogPtr dp, EventRecord *event, short *itemHit, void *storage)
{
#pragma unused(dp, event, itemHit, storage)
	return noErr;
}


/*
 *	ItemHit		-	handle an itemHit in a Preferences panel
 *
 */

static void ItemHit(PanelParameterBlock *pb, void *storage)
{
	ToolFrontEndPanelStatus** status = (ToolFrontEndPanelStatus**)storage;
	ToolFrontEndPref**	current = (ToolFrontEndPrefHandle) pb->currentPrefs;
	ToolFrontEndPref**	original = (ToolFrontEndPrefHandle) pb->originalPrefs;
	ControlHandle		ctrlHand;
	short				itemType;
	Rect				itemRect;
	short				value;
	Handle				itemHand;
	GrafPtr				save;
	
	GetPort(&save);
	SetPort(pb->dialog);
	
	switch (pb->itemHit - pb->baseItems)
	{
	case kExtensionPopupItem:
		GetDialogItem(pb->dialog, pb->itemHit, &itemType, (Handle *) &ctrlHand, &itemRect);
		value = GetControlValue(ctrlHand);
		switch (value)
		{
		case 1:
			// add new extension
			{
				Str15 extension;
				Byte state;
				Size size;
				
				// get the menu
				Handle controlData = (*ctrlHand)->contrlData;
				PopupPrivateDataHandle popupData = (PopupPrivateDataHandle)controlData;
				MenuHandle menu = (*popupData)->mHandle;
				
				// put up a dialog asking for an extension string
				if (!GetExtensionString(extension)) break;
				
				// XXX search the existing prefs for the string -- no dupes!
				
				// add a new entry at the end of the prefs
				state = HGetState((Handle)current);
				HLock((Handle)current);
				size = GetHandleSize((Handle)current);
				if (state & 0x4)
					SetResourceSize((Handle)current, size + sizeof(struct ExtensionEntry[1]));
				else
					SetHandleSize((Handle)current, size + sizeof(struct ExtensionEntry[1]));
				BlockMoveData(extension, 
								(*current)->extensions[(*current)->numExtensions].extension,
								extension[0] + 1);
				(*current)->extensions[(*current)->numExtensions].includeFileArg[0] = 0;
				(*current)->extensions[(*current)->numExtensions].scanner[0] = 0;
				(*current)->extensions[(*current)->numExtensions].commandTemplate[0] = 0;
				(*current)->numExtensions++;
				(*current)->scriptIncludeFile[0] = 0;
				HSetState((Handle)current, state);

				// add a new item at the end of the menu
				if ((*current)->numExtensions == 1)
					AppendMenu(menu, "\p(-");
				AppendMenu(menu, extension);
				
				// set the menu item and current status item
				(*status)->current = (*current)->numExtensions - 1;
				SetControlMaximum(ctrlHand, (*current)->numExtensions + 3);
				SetControlValue(ctrlHand, (*current)->numExtensions + 3);
				
				// redraw the dialog
				EnableMainItems(pb->dialog, pb->baseItems);
				DisableIncludeItems(pb->dialog, pb->baseItems);
				(*status)->enableIncludes = false;
				GetDialogItem(pb->dialog, pb->baseItems + kOverallBoxItem, &itemType, &itemHand, &itemRect);
				InvalRect(&itemRect);
				OptionsToDialog(pb->dialog, pb->currentPrefs, pb->baseItems, storage);
			}
			break;
			
		case 2:
			// remove current extension
			{
				Byte state;
				Size size;
				
				// get the menu
				Handle controlData = (*ctrlHand)->contrlData;
				PopupPrivateDataHandle popupData = (PopupPrivateDataHandle)controlData;
				MenuHandle menu = (*popupData)->mHandle;
				
				// delete the menu item
				DeleteMenuItem(menu, value);
				
				// shift and resize data in the handle
				--(*current)->numExtensions;
				if ((*current)->numExtensions != (*status)->current)
				{
					// shift back later entries
					BlockMoveData((*current)->extensions + (*status)->current + 1,
								  (*current)->extensions + (*status)->current,
								  sizeof(struct ExtensionEntry[1])
								  	* ((*current)->numExtensions - (*status)->current));
				}
				state = HGetState((Handle)current);
				size = GetHandleSize((Handle)current);
				if (state & 0x4)
					SetResourceSize((Handle)current, size - sizeof(struct ExtensionEntry[1]));
				else
					SetHandleSize((Handle)current, size - sizeof(struct ExtensionEntry[1]));
				
				// set the menu item and current status item
				if ((*current)->numExtensions == 0)
				{
					// last one went away
					DeleteMenuItem(menu, 3);	// delete separator
					DisableItem(menu, 2);		// delete remove item
					SetControlValue(ctrlHand, 1);
					(*status)->current = -1;
					
					// redraw the dialog
					DisableAllItems(pb->dialog, pb->baseItems);
					GetDialogItem(pb->dialog, pb->baseItems + kOverallBoxItem, &itemType, &itemHand, &itemRect);
					InvalRect(&itemRect);
				}
				else
				{
					(*status)->current = 0;		// select first item
					SetControlValue(ctrlHand, 4);
					
					// redraw the dialog
					(*status)->enableIncludes = ((*current)->extensions[(*status)->current].scanner[0] != 0);
					if ((*status)->enableIncludes)
						EnableIncludeItems(pb->dialog, pb->baseItems);
					else
						DisableIncludeItems(pb->dialog, pb->baseItems);
					GetDialogItem(pb->dialog, pb->baseItems + kOverallBoxItem, &itemType, &itemHand, &itemRect);
					InvalRect(&itemRect);
				}
				
				OptionsToDialog(pb->dialog, pb->currentPrefs, pb->baseItems, storage);
			}
			break;
			
		default:
			// select current extension
			DialogToOptions(pb->dialog, pb->currentPrefs, pb->baseItems, storage);
			(*status)->current = value - 4;
			
			// redraw the dialog
			(*status)->enableIncludes = ((*current)->extensions[(*status)->current].scanner[0] != 0);
			if ((*status)->enableIncludes)
				EnableIncludeItems(pb->dialog, pb->baseItems);
			else
				DisableIncludeItems(pb->dialog, pb->baseItems);
			GetDialogItem(pb->dialog, pb->baseItems + kOverallBoxItem, &itemType, &itemHand, &itemRect);
			InvalRect(&itemRect);
			OptionsToDialog(pb->dialog, pb->currentPrefs, pb->baseItems, storage);
			break;
		}
		break;

	default:
		DialogToOptions(pb->dialog, pb->currentPrefs, pb->baseItems, storage);
		break;
	}
	
	
	pb->canRevert	= !ComparePrefs(pb->originalPrefs, pb->currentPrefs);
	pb->canFactory	= !ComparePrefs(pb->factoryPrefs,  pb->currentPrefs);
	
	SetPort(save);
}

/*
 *	Validate	-	check if panel's changes require a recompile or relink
 *
 */

static void Validate(Handle original, Handle current, Boolean *recompile, Boolean *relink, Boolean *reset)
{
#pragma unused(original, current)
	
	// XXX Validate
	*recompile	= false;
	*relink		= false;
	*reset		= false;
}

/*
 *	GetPref		-	get a specified Preference setting for an AppleEvent request
 *
 */

static short GetPref(AEKeyword keyword, AEDesc *prefsDesc, Handle settings)
{
	ToolFrontEndPref	prefsData	= ** (ToolFrontEndPrefHandle) settings;
	DescType	anEnum;
	OSErr		err;
	
	switch (keyword)
	{
	// XXX get a specified Preference setting for an AppleEvent request
#if 0
	case prefsLN_GenerateSymFile:
		err = AECreateDesc(typeBoolean, &(*prefsData)->extensions[(*status)->current].linksym, sizeof(Boolean), prefsDesc);
		break;
		
	case prefsPR_ProjectType:
		switch ((*prefsData)->extensions[(*status)->current].projtype)
		{
		case kProjTypeApplication:	anEnum = enum_Project_Application;		break;
		case kProjTypeLibrary:		anEnum = enum_Project_Library;			break;
		case kProjTypeSharedLib:	anEnum = enum_Project_SharedLibrary;	break;
		case kProjTypeCodeResource:	anEnum = enum_Project_CodeResource;		break;
		case kProjTypeMPWTool:		anEnum = enum_Project_MPWTool;			break;
		default:					return (paramErr);
		}
		err = AECreateDesc(typeEnumeration, &anEnum, sizeof(anEnum), prefsDesc);
		break;
		
	case prefsPR_FileName:
		err = AECreateDesc(typeChar, (*prefsData)->extensions[(*status)->current].outfile+1, StrLength((*prefsData)->extensions[(*status)->current].outfile), prefsDesc);
		break;
#endif

	default:
		err = errAECantHandleClass;
		break;
	}
	
	return (err);
}

/*
 *	SetPref		-	set a specified Preference setting from an AppleEvent request
 *
 */

static short SetPref(AEKeyword keyword, const AEDesc *prefsDesc, Handle settings)
{
	ToolFrontEndPref	prefsData	= ** (ToolFrontEndPrefHandle) settings;
	AEDesc			toDesc	= { typeNull, NULL };
	OSErr			err		= noErr;
	Handle			dataHand;
	Size			textLength;
	DescType		anEnum;
	
	switch (keyword)
	{
	// XXX set a specified Preference setting from an AppleEvent request
#if 0
	case prefsLN_GenerateSymFile:
		if (prefsDesc->descriptorType == typeBoolean)
		{
			dataHand = prefsDesc->dataHandle;
		}
		else
		{
			err = AECoerceDesc(prefsDesc, typeBoolean, &toDesc);
			if (err == noErr)
				dataHand = toDesc.dataHandle;
		}
		if (err == noErr)
		{
			(*prefsData)->extensions[(*status)->current].linksym = ** (Boolean **) dataHand;
		}
		break;
		
	case prefsPR_ProjectType:
		if (prefsDesc->descriptorType != typeEnumeration)
		{
			err = errAETypeError;
			break;
		}

		anEnum = ** (DescType **) prefsDesc->dataHandle;
		
		switch (anEnum)
		{
		case enum_Project_Application:		(*prefsData)->extensions[(*status)->current].projtype = kProjTypeApplication;	break;
		case enum_Project_Library:			(*prefsData)->extensions[(*status)->current].projtype = kProjTypeLibrary;		break;
		case enum_Project_SharedLibrary:	(*prefsData)->extensions[(*status)->current].projtype = kProjTypeSharedLib;	break;
		case enum_Project_CodeResource:		(*prefsData)->extensions[(*status)->current].projtype = kProjTypeCodeResource;	break;
		case enum_Project_MPWTool:			(*prefsData)->extensions[(*status)->current].projtype = kProjTypeMPWTool;		break;
		default:							return (errAECoercionFail);
		}
		break;
		
	case prefsPR_FileName:
		if (prefsDesc->descriptorType == typeChar)
		{
			dataHand = prefsDesc->dataHandle;
		}
		else
		{
			err = AECoerceDesc(prefsDesc, typeChar, &toDesc);
			if (err == noErr)
				dataHand = toDesc.dataHandle;
		}
		if (err == noErr)
		{
			textLength = GetHandleSize(dataHand);
			if (textLength > sizeof((*prefsData)->extensions[(*status)->current].outfile) - 1)
				textLength = sizeof((*prefsData)->extensions[(*status)->current].outfile) - 1;
			BlockMoveData(*dataHand, (*prefsData)->extensions[(*status)->current].outfile+1, textLength);
			(*prefsData)->extensions[(*status)->current].outfile[0] = textLength;
		}
		break;
#endif

	default:
		err = errAECantHandleClass;
		break;
	}
	
	if (err == noErr)
	{
		** (ToolFrontEndPrefHandle) settings = prefsData;
	}
	
	AEDisposeDesc(&toDesc);
	
	return (err);
}

/*
 *	GetFactory	-	retrieve factory settings
 *
 */

static short GetFactory(Handle settings, DialogPtr dp)
{
#pragma unused(dp)

	OSErr err;
	SetHandleSize(settings, sizeof(ToolFrontEndPref));
	err = MemError();
	if (err == noErr)
	{
		int i;
		char *c;
		
		for (i = sizeof(ToolFrontEndPref), c = (char*)*settings; i > 0; i--)
			*c++ = 0;
	}
	return (err);
}

/*
 *	UpdatePref	-	"upgrade" a pref to the current version
 *
 */

static short UpdatePref(Handle settings)
{
	ToolFrontEndPrefHandle	prefsHand	= (ToolFrontEndPrefHandle) settings;
	OSErr err = noErr;
	
	switch ((*prefsHand)->version)
	{
	case PTOOLFRONTENDVERSION:
		break;
	default:
		err = kBadPrefVersion;
		break;
	}
	
	return (err);
}


static Boolean ComparePrefs(Handle prefsHand1, Handle prefsHand2)
{
	ToolFrontEndPrefHandle	prefs1	= (ToolFrontEndPrefHandle) prefsHand1;
	ToolFrontEndPrefHandle	prefs2	= (ToolFrontEndPrefHandle) prefsHand2;
	Boolean differ = false;
	Byte state1, state2;
	
	state1 = HGetState(prefsHand1);
	HLock(prefsHand1);
	state2 = HGetState(prefsHand2);
	HLock(prefsHand2);
	
	if ((*prefs1)->numExtensions != (*prefs2)->numExtensions)
	{
		differ = true;
	}
	else
	{
		int i;
		differ = EqualString((*prefs1)->scriptIncludeFile, (*prefs2)->scriptIncludeFile, false, true);
		for (i = 0; !differ && i < (*prefs1)->numExtensions; i++)
		{
			differ = (	EqualString((*prefs1)->extensions[i].extension, (*prefs2)->extensions[i].extension, false, true)
					&&  EqualString((*prefs1)->extensions[i].scanner, (*prefs2)->extensions[i].scanner, false, true)
					&&  EqualString((*prefs1)->extensions[i].includeFileArg, (*prefs2)->extensions[i].includeFileArg, false, true)
					&&  EqualString((*prefs1)->extensions[i].commandTemplate, (*prefs2)->extensions[i].commandTemplate, false, true));
		}
	}
	
	HSetState(prefsHand1, state1);
	HSetState(prefsHand2, state2);
	
	return !differ;
}


void EnableMainItems(DialogPtr dp, short baseItems)
{
	ControlHandle		ctrlHand;
	short				itemType;
	Rect				itemRect;
	Handle				itemHand;
	
	GetDialogItem(dp, baseItems + kIncludeScannerItem, &itemType, (Handle*)&ctrlHand, &itemRect);
	HiliteControl(ctrlHand, 0);
	GetDialogItem(dp, baseItems + kCommandItem, &itemType, &itemHand, &itemRect);
	SetDialogItem(dp, baseItems + kCommandItem, editText, itemHand, &itemRect);
	FixEditField(dp);
}


void EnableIncludeItems(DialogPtr dp, short baseItems)
{
	ControlHandle		ctrlHand;
	short				itemType;
	Rect				itemRect;
	Handle				itemHand;
	
	GetDialogItem(dp, baseItems + kIncludeScannerItem, &itemType, (Handle*)&ctrlHand, &itemRect);
	HiliteControl(ctrlHand, 0);
	GetDialogItem(dp, baseItems + kIncludePathItem, &itemType, &itemHand, &itemRect);
	SetDialogItem(dp, baseItems + kIncludePathItem, editText, itemHand, &itemRect);
	FixEditField(dp);
}


void DisableAllItems(DialogPtr dp, short baseItems)
{
	short				itemType;
	Rect				itemRect;
	Handle				itemHand;
	
	GetDialogItem(dp, baseItems + kCommandItem, &itemType, &itemHand, &itemRect);
	SetDialogItem(dp, baseItems + kCommandItem, statText + itemDisable, itemHand, &itemRect);
	DisableIncludeItems(dp, baseItems);
	FixEditField(dp);
}


void DisableIncludeItems(DialogPtr dp, short baseItems)
{
	ControlHandle		ctrlHand;
	short				itemType;
	Rect				itemRect;
	Handle				itemHand;
	
	GetDialogItem(dp, baseItems + kIncludeScannerItem, &itemType, (Handle*)&ctrlHand, &itemRect);
	HiliteControl(ctrlHand, 255);
	GetDialogItem(dp, baseItems + kIncludePathItem, &itemType, &itemHand, &itemRect);
	SetDialogItem(dp, baseItems + kIncludePathItem, statText + itemDisable, itemHand, &itemRect);
	FixEditField(dp);
}


void FixEditField(DialogPtr dp)
{
	DialogPeek peek = (DialogPeek)dp;
	short				itemType;
	Rect				itemRect;
	Handle				itemHand;
	short				editItem = peek->editField + 1;
	
	if (editItem > 0)
	{
		// verify that the item deserves the kleyboard focus; if not, go somewhere else
		GetDialogItem(dp, editItem, &itemType, &itemHand, &itemRect);
		if (itemType != editText)
		{
			int i, lim;
			Boolean found = false;
			
			SelectDialogItemText(dp, editItem, 0, 0);
			for (i = 1, lim = CountDITL(dp); i <= lim; i++)
			{
				GetDialogItem(dp, editItem, &itemType, &itemHand, &itemRect);
				if (itemType == editText)
				{
					SelectDialogItemText(dp, i, 0, 32767);
					found = true;
					break;
				}
			}
			if (!found)
			{
				// there are no edit text items
				peek->editField = -1;
			}
		}
	}
	else
	{
		// no keyboard focus -- see if there should be one
		int i, lim;
		for (i = 1, lim = CountDITL(dp); i <= lim; i++)
		{
			GetDialogItem(dp, editItem, &itemType, &itemHand, &itemRect);
			if (itemType == editText)
			{
				SelectDialogItemText(dp, i, 0, 32767);
				break;
			}
		}
	}
}


Boolean	GetExtensionString(StringPtr extension)
{
	short				itemHit;
	short				itemType;
	Rect				itemRect;
	Handle				itemHand;
	Boolean				done, result;
	
	DialogPtr dialog = GetNewDialog(kGetExtensionDialog, NULL, (WindowPtr)-1);
	SelectDialogItemText(dialog, kExtensionTextItem, 0, 32767);
	done = false;
	do
	{
		ModalDialog(NULL, &itemHit);
		switch (itemHit)
		{
		case ok:
			GetDialogItem(dialog, kExtensionTextItem, &itemType, &itemHand, &itemRect);
			GetDialogItemText(itemHand, extension);
			result = true;
			done = true;
			break;
		case cancel:
			result = false;
			done = true;
			break;
		}
	} while (!done);
	DisposeDialog(dialog);
		
	return result;
}
